<?php 

namespace WebServiceCall\Service;

use WebServiceCall\Service\Interfaces\ISales;

use WebServiceCall\Repository\DB\Sales as SalesRepo;
use WebServiceCall\Repository\DB\Adquirente as AdquirenteRepo;
use WebServiceCall\Repository\DB\ProdutoAdq as ProdutoAdqRepo;

use WebServiceCall\Repository\WS\Data\SalesParams;

class Sales extends ABaseWSService implements ISales {
    private const SERVICE_NAME = 'Vendas';
    private $salesRepo;
    private $adquirenteRepo;
    private $produtoAdqRepo;
    
    public function __construct($connParams, $initialDate, $endDate) {
        parent::__construct($connParams, $initialDate, $endDate, self::SERVICE_NAME);
    
        $this->setSalesRepo(new SalesRepo ());
        $this->setAdquirenteRepo(new AdquirenteRepo ());
        $this->setProdutoAdqRepo(new ProdutoAdqRepo ());
    }
    
    private function setSalesRepo($salesRepo) {
        $this->salesRepo = $salesRepo;
    }
    
    private function setAdquirenteRepo($adquirenteRepo) {
        $this->adquirenteRepo = $adquirenteRepo;
    }
    
    private function setProdutoAdqRepo($produtoAdqRepo) {
        $this->produtoAdqRepo = $produtoAdqRepo;
    }
    
    // @Override
    public function importSalesFromWebService() {
        parent::importDataFromWebService();
    }
    
    // @Override
    protected function getSoapParams($connParam) {
        return new SalesParams(
            $connParam->getIdEmpresa(), 
            $connParam->getPass(),
            $this->getInitialDate(),
            $this->getFinalDate()
        );
    }
    
    // @Override
    protected function setInsertParams($sale) {
        return $this->setSaleParams($sale);
    }
    
    // @Override
    protected function saveDataInDB($sale) {
        return $this->saveSale($sale);
    }
    
    // @Override
    protected function commit() {
        return $this->salesRepo->commit();
    }
    
    // @Override
    protected function rollback() {
        return $this->salesRepo->rollback();
    }
    
    private function saveSale($sale) {
        $saleParams = [];

        $nsu = $sale[':Nsu'];
        $autorizacao = $sale[':Autorizacao'];
        $dtVenda = $sale[':DataVenda'];
        
        // Chama o método que valida se a venda atual já está inserida
        $canInsert = $this->salesRepo->isSaleInserted($nsu, $autorizacao, $dtVenda);

        if ($canInsert) {
            // Os campos Pago e Cancelamento do relatório são ignorados 
            // pois a função deles foi incorporada em uma única coluna (IDSTATUS) no nosso sistema. 
            // O campo CNPJ não é parte das colunas da tabela RETORIGVENDA
            foreach ($sale as $key => $value) {
                if ($this->isAValidInsertSaleParamKey($key) ) {
                    $saleParams[$key] = $value;
                    
                    if ($this->doesntAdquirenteInformsCDLOTE($key, $value) ) {
                        $saleParams[':ResumoVenda'] = NULL;
                    }
                }
            }

            return $this->salesRepo->insertSale($saleParams);
        }
        
        return true; // No errors, but register already exists.
    }
    
    private function setSaleParams($sale) {
        $nrAdquirente = $this->adquirenteRepo->getNextAdquirenteID($sale[':Adquirente']);
        $nrProdutoAdq = $this->produtoAdqRepo->getNexProdutoID($sale[':CodigoProduto']);
        
        // Define os dados do Adquirente e do Produto para a venda selecionada        
        $sale[':NRADQUIRENTE']   = $nrAdquirente;
        $sale[':NRPRODUTOADQ']   = $nrProdutoAdq;
        
        // Formatação dos valores da venda selecionada. 
        $sale[":ValorBruto"]     = $this->getFormattedSaleMoneyValue($sale[":ValorBruto"]);
        $sale[":PercentualTaxa"] = $this->getFormattedSaleMoneyValue($sale[":PercentualTaxa"]);
        $sale[":ValorTaxa"]      = $this->getFormattedSaleMoneyValue($sale[":ValorTaxa"]);
        $sale[":ValorLiquido"]   = $this->getFormattedSaleMoneyValue($sale[":ValorLiquido"]);
        $sale[":ValorPago"]      = $this->getFormattedSaleMoneyValue($sale[":ValorPago"]);

        // Concatena a data da venda ao hórario da transação para ser
        // compatível com o formato date do banco de dados oracle
        $sale[':HoraTransacao'] = $sale[':DataVenda'] . ' ' . $sale[':HoraTransacao'];

        // Converte os valores do relatório para caracteres, 
        // de forma que esses valores sejam compatíveis com os valores
        // possíveis para as colunas no BD
        $sale[':Rejeitado'] = $sale[':Rejeitado'] === 'true' ? 'S' : 'N';
        $sale[':Cancelado'] = $sale[':Cancelado'] === 'true' ? 'S' : 'N';
        $sale[':Pago']      = $sale[':Pago'] === 'true' ? 'S' : 'N';

        switch ($sale[':Status']) {
            case '1':
                $sale[':Status'] = 'P'; /* Pago */
                break;
            case '2':
                $sale[':Status'] = 'A'; /* Atrasado */
                break;
            case '3':
                $sale[':Status'] = 'C'; /* Cancelado */
                break;
            case '4':
                $sale[':Status'] = 'G'; /* Aguardando Pagamento */
                break;
        }

        switch ($sale[':CodigoModalidade']) {
            case '0':
                $sale[':CodigoModalidade'] = 'D'; /* Débito */
                break;
            case '1':
                $sale[':CodigoModalidade'] = 'C'; /* Credito a Vista */
                break;
            case '2':
                $sale[':CodigoModalidade'] = 'P'; /* Crédito Parcelado */
                break;
            case '3':
                $sale[':CodigoModalidade'] = 'B'; /* Beneficio */
                break;
        }
        
        return $sale;
    }
    
    private function getFormattedSaleMoneyValue($value) {
        return str_replace(".", ",", floatval($value));
    }
    
    private function isAValidInsertSaleParamKey($key) {
        return  !($key === ':Pago' ||
                  $key === ':Cnpj' ||
                  $key === ':Tid' || 
                  $key === ':Observacao' ||
                  $key === ':Cancelamento');
    }
    
    /* A Ticket e a VR não informam código do lote*/
    private function doesntAdquirenteInformsCDLOTE($key, $value) {
        return ($key === ':Adquirente' && 
               ($value === 'Ticket' || $value === 'VR' || $value === 'Valecard'));
    }
}